import type * as ts from 'typescript'

export class TypeScriptBinaryLoader {
  private tsBinary?: typeof ts

  public load(): typeof ts {
    if (this.tsBinary) {
      return this.tsBinary
    }

    try {
      const tsBinaryPath = require.resolve('typescript', {
        paths: [process.cwd(), ...this.getModulePaths()]
      })
      // eslint-disable-next-line @typescript-eslint/no-var-requires
      const tsBinary = require(tsBinaryPath)
      this.tsBinary = tsBinary
      return tsBinary
    } catch {
      throw new Error(
        'TypeScript could not be found! Please, install "typescript" package.'
      )
    }
  }

  public getModulePaths() {
    const modulePaths = module.paths.slice(2, module.paths.length)
    const packageDeps = modulePaths.slice(0, 3)
    return [
      ...packageDeps.reverse(),
      ...modulePaths.slice(3, modulePaths.length).reverse()
    ]
  }
}
